"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const listr_1 = __importDefault(require("listr"));
const lodash_1 = require("lodash");
const path_1 = __importDefault(require("path"));
const async_1 = require("../../utils/async");
const contentful_clients_1 = require("../../utils/contentful-clients");
const copyright_1 = require("../../utils/copyright");
const cursor_pagninate_1 = require("../../utils/cursor-pagninate");
const fs_1 = require("../../utils/fs");
const headers_1 = require("../../utils/headers");
const log_1 = require("../../utils/log");
module.exports.command = 'export';
module.exports.desc = 'export your organization entities';
module.exports.builder = (yargs) => {
    return yargs
        .usage('Usage: contentful organization export')
        .option('management-token', {
        alias: 'mt',
        describe: 'Contentful management API token',
        type: 'string'
    })
        .option('organization-id', {
        alias: 'oid',
        describe: 'ID of Organization with source data',
        type: 'string',
        demandOption: true
    })
        .option('header', {
        alias: 'H',
        type: 'string',
        describe: 'Pass an additional HTTP Header'
    })
        .option('output-file', {
        alias: 'o',
        type: 'string',
        describe: 'Output file. It defaults to ./data/<timestamp>-<organization-id>.json'
    })
        .option('save-file', {
        describe: 'Save the export as a json file',
        type: 'boolean',
        default: true
    })
        .option('silent', {
        alias: 'S',
        type: 'boolean',
        describe: 'Suppress any log output',
        default: false
    })
        .epilog([
        'See more at:',
        'https://github.com/contentful/contentful-cli/tree/master/docs/organization/export',
        copyright_1.copyright
    ].join('\n'));
};
function organizationExport(_a) {
    return __awaiter(this, arguments, void 0, function* ({ context, header, organizationId, outputFile, saveFile, silent }) {
        const { managementToken } = context;
        const client = yield (0, contentful_clients_1.createPlainClient)({
            accessToken: managementToken,
            feature: 'organization-export',
            headers: (0, headers_1.getHeadersFromOption)(header),
            throttle: 8,
            logHandler: lodash_1.noop
        });
        const outputTarget = (0, fs_1.getPath)(outputFile || path_1.default.join('data', `${Date.now()}-${organizationId}.json`));
        yield (0, fs_1.ensureDir)(path_1.default.dirname(outputTarget));
        const tasks = new listr_1.default([
            {
                title: 'Exporting Organization',
                task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                    return new listr_1.default([
                        {
                            title: 'Exporting Concepts',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                ctx.taxonomy.concepts = yield (0, cursor_pagninate_1.cursorPaginate)({
                                    queryPage: pageUrl => client.concept.getMany({
                                        organizationId,
                                        query: { pageUrl }
                                    })
                                });
                            })
                        },
                        {
                            title: 'Exporting Concept Schemes',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                ctx.taxonomy.conceptSchemes = yield (0, cursor_pagninate_1.cursorPaginate)({
                                    queryPage: pageUrl => client.conceptScheme.getMany({
                                        organizationId,
                                        query: { pageUrl }
                                    })
                                });
                            })
                        }
                    ]);
                })
            }
        ], { renderer: silent ? 'silent' : 'default' });
        const result = yield tasks.run({
            taxonomy: { concepts: [], conceptSchemes: [] }
        });
        if (saveFile) {
            yield (0, fs_1.writeFileP)(outputTarget, JSON.stringify(result, null, 2));
        }
        else {
            (0, log_1.log)(JSON.stringify(result, null, 2));
        }
        !silent && (0, log_1.success)(`✅ Organization data exported to ${outputTarget}`);
    });
}
module.exports.organizationExport = organizationExport;
module.exports.handler = (0, async_1.handleAsyncError)(organizationExport);
exports.default = organizationExport;
