"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultLocale = void 0;
/* eslint-disable @typescript-eslint/no-empty-function */
const listr_1 = __importDefault(require("listr"));
const lodash_1 = require("lodash");
const path_1 = __importDefault(require("path"));
const async_1 = require("../../utils/async");
const contentful_clients_1 = require("../../utils/contentful-clients");
const cursor_pagninate_1 = require("../../utils/cursor-pagninate");
const fs_1 = require("../../utils/fs");
const headers_1 = require("../../utils/headers");
const log_1 = require("../../utils/log");
const Papa = __importStar(require("papaparse"));
const taxonomy_1 = require("./utils/taxonomy");
module.exports.command = 'taxonomy-transform';
module.exports.desc =
    'transform taxonomy from external format to contentful json format';
module.exports.builder = (yargs) => {
    return yargs
        .usage('Usage: contentful organization taxonomy-transform')
        .option('management-token', {
        alias: 'mt',
        describe: 'Contentful management API token',
        type: 'string'
    })
        .option('organization-id', {
        alias: 'oid',
        describe: 'ID of Organization with source data',
        type: 'string',
        demandOption: true
    })
        .option('header', {
        alias: 'H',
        type: 'string',
        describe: 'Pass an additional HTTP Header'
    })
        .option('output-file', {
        alias: 'o',
        type: 'string',
        describe: 'Output file. It defaults to ./data/<timestamp>-<organization-id>-transformed.json'
    })
        .option('transform-script', {
        alias: 't',
        describe: 'Script used to transform the taxonomy data',
        type: 'string',
        demandOption: true
    })
        .option('save-file', {
        describe: 'Save the transformed taxonomies as a json file',
        type: 'boolean',
        default: true
    })
        .option('silent', {
        alias: 's',
        type: 'boolean',
        describe: 'Suppress any log output',
        default: false
    });
};
exports.defaultLocale = 'en-US';
const transformContext = {
    csv: {
        parse: Papa.parse
    },
    fs: {
        readFile: fs_1.readFileP,
        cwd: process.cwd
    },
    taxonomy: new taxonomy_1.Taxonomy()
};
function taxonomyTransform(_a) {
    return __awaiter(this, arguments, void 0, function* ({ context, header, organizationId, outputFile, saveFile, transformScript, silent }) {
        const { managementToken } = context;
        const client = yield (0, contentful_clients_1.createPlainClient)({
            accessToken: managementToken,
            feature: 'taxonomy-transform',
            headers: (0, headers_1.getHeadersFromOption)(header),
            throttle: 8,
            logHandler: lodash_1.noop
        });
        const outputTarget = (0, fs_1.getPath)(outputFile ||
            path_1.default.join('data', `${Date.now()}-${organizationId}-transformed.json`));
        yield (0, fs_1.ensureDir)(path_1.default.dirname(outputTarget));
        const tasks = new listr_1.default([
            {
                title: 'Transforming taxonomy data',
                task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                    return new listr_1.default([
                        {
                            title: 'Exporting Concepts',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                ctx.taxonomy.setExistingConcepts(yield (0, cursor_pagninate_1.cursorPaginate)({
                                    queryPage: pageUrl => client.concept.getMany({
                                        organizationId,
                                        query: { pageUrl }
                                    })
                                }));
                            })
                        },
                        {
                            title: 'Exporting Concept Schemes',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                ctx.taxonomy.setExistingConceptSchemes(yield (0, cursor_pagninate_1.cursorPaginate)({
                                    queryPage: pageUrl => client.conceptScheme.getMany({
                                        organizationId,
                                        query: { pageUrl }
                                    })
                                }));
                            })
                        },
                        {
                            title: 'Running transform script',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                const filePath = path_1.default.resolve(process.cwd(), transformScript);
                                const transform = yield Promise.resolve(`${filePath}`).then(s => __importStar(require(s)));
                                yield transform.default(ctx);
                            })
                        }
                    ]);
                })
            }
        ], { renderer: silent ? 'silent' : 'default' });
        yield tasks.run(transformContext);
        const result = transformContext.taxonomy.toJson();
        if (saveFile) {
            yield (0, fs_1.writeFileP)(outputTarget, JSON.stringify(result, null, 2));
            !silent &&
                (0, log_1.success)(`✅ Data transformed successfully and saved to ${outputTarget}`);
        }
        else {
            (0, log_1.log)(JSON.stringify(result, null, 2));
            !silent && (0, log_1.success)(`✅ Data transformed successfully`);
        }
    });
}
module.exports.taxonomyTransform = taxonomyTransform;
module.exports.handler = (0, async_1.handleAsyncError)(taxonomyTransform);
exports.default = taxonomyTransform;
