"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConceptScheme = void 0;
const taxonomy_transform_1 = require("../taxonomy-transform");
class ConceptScheme {
    constructor(id, init) {
        this.model = Object.assign({ id }, init);
    }
    toJson() {
        return structuredClone(this.model);
    }
    setUri(uri) {
        this.model.uri = uri;
        return this;
    }
    setDefinition(definition) {
        if (!this.model.definition) {
            this.model.definition = {};
        }
        this.model.definition[taxonomy_transform_1.defaultLocale] = definition;
        return this;
    }
    setPrefLabel(prefLabel) {
        if (!this.model.prefLabel) {
            this.model.prefLabel = {};
        }
        this.model.prefLabel[taxonomy_transform_1.defaultLocale] = prefLabel;
        return this;
    }
    addTopConcept(conceptId) {
        if (!this.model.topConcepts) {
            this.model.topConcepts = [];
        }
        this.model.topConcepts.push({
            sys: {
                id: conceptId,
                type: 'Link',
                linkType: 'TaxonomyConcept'
            }
        });
        this.model.totalConcepts = (this.model.totalConcepts || 0) + 1;
        return this;
    }
    removeTopConcept(conceptId) {
        if (this.model.topConcepts) {
            this.model.topConcepts = this.model.topConcepts.filter(concept => concept.sys.id !== conceptId);
        }
        if (this.model.concepts) {
            this.model.concepts = this.model.concepts.filter(concept => concept.sys.id !== conceptId);
        }
        this.model.totalConcepts = this.model.totalConcepts
            ? this.model.totalConcepts - 1
            : 0;
        return this;
    }
    addConcept(conceptId) {
        if (!this.model.concepts) {
            this.model.concepts = [];
        }
        this.model.concepts.push({
            sys: {
                id: conceptId,
                type: 'Link',
                linkType: 'TaxonomyConcept'
            }
        });
        this.model.totalConcepts = (this.model.totalConcepts || 0) + 1;
        return this;
    }
    removeConcept(conceptId) {
        if (this.model.concepts) {
            if (!this.model.concepts.find(concept => concept.sys.id === conceptId)) {
                throw new Error(`Concept with id ${conceptId} does not exist in Concept Scheme`);
            }
            this.model.concepts = this.model.concepts.filter(concept => concept.sys.id !== conceptId);
            this.model.totalConcepts = this.model.totalConcepts
                ? this.model.totalConcepts - 1
                : 0;
        }
        return this;
    }
}
exports.ConceptScheme = ConceptScheme;
