"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Concept = void 0;
const taxonomy_transform_1 = require("../taxonomy-transform");
class Concept {
    constructor(id, init) {
        this.model = Object.assign({ id }, init);
    }
    toJson() {
        return structuredClone(this.model);
    }
    setUri(uri) {
        this.model.uri = uri;
        return this;
    }
    setPrefLabel(prefLabel) {
        if (!this.model.prefLabel) {
            this.model.prefLabel = {};
        }
        this.model.prefLabel[taxonomy_transform_1.defaultLocale] = prefLabel;
        return this;
    }
    setDefinition(definition) {
        if (!this.model.definition) {
            this.model.definition = {};
        }
        this.model.definition[taxonomy_transform_1.defaultLocale] = definition;
        return this;
    }
    addAltLabel(altLabel) {
        if (!this.model.altLabels) {
            this.model.altLabels = { [taxonomy_transform_1.defaultLocale]: [] };
        }
        this.model.altLabels[taxonomy_transform_1.defaultLocale].push(altLabel);
        return this;
    }
    removeAltLabel(altLabel) {
        if (this.model.altLabels) {
            this.model.altLabels[taxonomy_transform_1.defaultLocale] = this.model.altLabels[taxonomy_transform_1.defaultLocale].filter(label => label !== altLabel);
        }
        return this;
    }
    addHiddenLabel(hiddenLabel) {
        if (!this.model.hiddenLabels) {
            this.model.hiddenLabels = { [taxonomy_transform_1.defaultLocale]: [] };
        }
        this.model.hiddenLabels[taxonomy_transform_1.defaultLocale].push(hiddenLabel);
        return this;
    }
    removeHiddenLabel(hiddenLabel) {
        if (this.model.hiddenLabels) {
            this.model.hiddenLabels[taxonomy_transform_1.defaultLocale] = this.model.hiddenLabels[taxonomy_transform_1.defaultLocale].filter(label => label !== hiddenLabel);
        }
        return this;
    }
    setEditorialNote(editorialNote) {
        if (!this.model.editorialNote) {
            this.model.editorialNote = {};
        }
        this.model.editorialNote[taxonomy_transform_1.defaultLocale] = editorialNote;
        return this;
    }
    setHistoryNote(historyNote) {
        if (!this.model.historyNote) {
            this.model.historyNote = {};
        }
        this.model.historyNote[taxonomy_transform_1.defaultLocale] = historyNote;
        return this;
    }
    setExample(example) {
        if (!this.model.example) {
            this.model.example = {};
        }
        this.model.example[taxonomy_transform_1.defaultLocale] = example;
        return this;
    }
    setNote(note) {
        if (!this.model.note) {
            this.model.note = {};
        }
        this.model.note[taxonomy_transform_1.defaultLocale] = note;
        return this;
    }
    setScopeNote(scopeNote) {
        if (!this.model.scopeNote) {
            this.model.scopeNote = {};
        }
        this.model.scopeNote[taxonomy_transform_1.defaultLocale] = scopeNote;
        return this;
    }
    addNotation(notation) {
        if (!this.model.notations) {
            this.model.notations = [];
        }
        this.model.notations.push(notation);
        return this;
    }
    removeNotation(notation) {
        if (this.model.notations) {
            this.model.notations = this.model.notations.filter(n => n !== notation);
        }
        return this;
    }
    addBroader(conceptId) {
        if (!this.model.broader) {
            this.model.broader = [];
        }
        this.model.broader.push({
            sys: {
                id: conceptId,
                type: 'Link',
                linkType: 'TaxonomyConcept'
            }
        });
        return this;
    }
    removeBroader(conceptId) {
        if (this.model.broader) {
            this.model.broader = this.model.broader.filter(concept => concept.sys.id !== conceptId);
        }
        return this;
    }
    addRelated(conceptId) {
        if (!this.model.related) {
            this.model.related = [];
        }
        this.model.related.push({
            sys: {
                id: conceptId,
                type: 'Link',
                linkType: 'TaxonomyConcept'
            }
        });
        return this;
    }
    removeRelated(conceptId) {
        if (this.model.related) {
            this.model.related = this.model.related.filter(concept => concept.sys.id !== conceptId);
        }
        return this;
    }
}
exports.Concept = Concept;
