"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Taxonomy = void 0;
const concept_1 = require("./concept");
const concept_scheme_1 = require("./concept-scheme");
class Taxonomy {
    constructor() {
        this.concepts = {
            existing: [],
            upsert: []
        };
        this.conceptSchemes = {
            existing: [],
            upsert: []
        };
    }
    toJson() {
        return {
            concepts: this.concepts.upsert.map(concept => concept.toJson()),
            conceptSchemes: this.conceptSchemes.upsert.map(conceptScheme => conceptScheme.toJson())
        };
    }
    setExistingConcepts(concepts) {
        this.concepts.existing = concepts.map(concept => new concept_1.Concept(concept.sys.id, concept));
    }
    setExistingConceptSchemes(conceptSchemes) {
        this.conceptSchemes.existing = conceptSchemes.map(conceptScheme => new concept_scheme_1.ConceptScheme(conceptScheme.sys.id, conceptScheme));
    }
    addConcept(id, init) {
        if (this.concepts.existing.find(concept => concept.toJson().id === id)) {
            return null;
        }
        if (this.concepts.upsert.find(concept => concept.toJson().id === id)) {
            return null;
        }
        const concept = new concept_1.Concept(id, init);
        this.concepts.upsert.push(concept);
        return concept;
    }
    getConcept(id) {
        const existingConcept = this.concepts.existing.find(concept => concept.toJson().id === id);
        const addedConcept = this.concepts.upsert.find(concept => concept.toJson().id === id);
        if (!existingConcept && !addedConcept) {
            return null;
        }
        if (existingConcept && !addedConcept) {
            this.concepts.upsert.push(existingConcept);
        }
        return addedConcept || existingConcept;
    }
    addConceptScheme(id, init) {
        if (this.conceptSchemes.existing.find(conceptScheme => conceptScheme.toJson().id === id)) {
            return null;
        }
        if (this.conceptSchemes.upsert.find(conceptScheme => conceptScheme.toJson().id === id)) {
            return null;
        }
        const conceptScheme = new concept_scheme_1.ConceptScheme(id, init);
        this.conceptSchemes.upsert.push(conceptScheme);
        return conceptScheme;
    }
    getConceptScheme(id) {
        const existingConceptScheme = this.conceptSchemes.existing.find(conceptScheme => conceptScheme.toJson().id === id);
        const addedConceptScheme = this.conceptSchemes.upsert.find(conceptScheme => conceptScheme.toJson().id === id);
        if (!existingConceptScheme && !addedConceptScheme) {
            return null;
        }
        if (existingConceptScheme && !addedConceptScheme) {
            this.conceptSchemes.upsert.push(existingConceptScheme);
        }
        return addedConceptScheme || existingConceptScheme;
    }
}
exports.Taxonomy = Taxonomy;
